static char *rcsid="$Id: glsimplexview.c,v 1.3 2003/03/28 16:47:47 fuhrmann Exp $";

#include "glsimplexview.h"
#include "glmesh.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

struct gl_simplex_mesh_struct
{
  int dim;
  int nn;
  int nc;
  int *sxlist;
  double *xc,*yc,*zc;
  double xmin,xmax,ymin,ymax,zmin,zmax;
  double *cellflux;
  double *nodefunc;
  int owncoord,owncflux,ownnfunc;
  char axisname[4][64];
  glMesh glm;
  glRenderer rnd;
};



GLSimplexMesh glsxvCreate(int dim,int nn, int nc, int *sxlist,
		     double *xc, double *yc, double *zc)
{
  GLSimplexMesh m;
  m=calloc(1,sizeof(struct gl_simplex_mesh_struct));
  m->nc=nc;
  m->nn=nn;
  m->xc=xc;
  m->yc=yc;
  m->zc=zc;
  m->sxlist=sxlist;
  m->owncoord=0;
  m->ownnfunc=0;
  m->owncflux=0;
  m->xmin=1;m->xmax=0;
  m->ymin=1;m->ymax=0;
  m->zmin=1;m->zmax=0;
  strcpy(m->axisname[0],"x");
  strcpy(m->axisname[1],"y");
  strcpy(m->axisname[2],"z");
  strcpy(m->axisname[3],"f");
  m->dim=dim;
  m->rnd=0;
  return m;
}



void glsxvDestroy(GLSimplexMesh m)
{
  if (m->ownnfunc&&m->nodefunc)free(m->nodefunc);
  if (m->owncflux&&m->cellflux)free(m->cellflux);
  if (m->owncoord) 
    {
      free(m->xc);
      free(m->yc);
      free(m->zc);
    }
  if (m->rnd) glrDestroy(m->rnd);
  free(m);
}

void glsxvSetBox(GLSimplexMesh m, double xmin, double xmax, double ymin, double ymax,double zmin,double zmax)
{
  m->xmin=xmin;
  m->xmax=xmax;

  m->ymin=ymin;
  m->ymax=ymax;

  m->zmin=zmin;
  m->zmax=zmax;


}

void glsxvCalcNodeFunc(GLSimplexMesh m,glsxvFunc f, double t)
{
  register int i,j,k,l;
  double*xc=m->xc;
  double*yc=m->yc;
  double*zc=m->zc;
  double*v;
  if (m->nodefunc==0) m->nodefunc=calloc(m->nn,sizeof(double));
  m->ownnfunc=1;
  v=m->nodefunc;
  for (i=1;i<=m->nn;i++)
    v[i]=f(xc[i],yc[i],zc[i],t);
}

void glsxvCalcCellFlux(GLSimplexMesh m, glsxvFlux flux,double t)
{
  double x,y,z;
  register int i,id,*idx;
  double*xc=m->xc;
  double*yc=m->yc;
  double*zc=m->zc;
  double*flx;
  m->owncflux=1;
  if (m->cellflux==0)
    {
	m->cellflux=calloc(m->dim*m->nc+10,sizeof(double));
    }
  

  for (i=1;i<=m->nc;i++)
    {
      idx=&m->sxlist[i*(m->dim+1)];
      flx=&m->cellflux[i*m->dim];
      x=y=z=0;
      for (id=1;id<=m->dim+1;id++)
	{
	  x+=xc[idx[i]];
	  y+=yc[idx[i]];
	  z+=zc[idx[i]];
	}
      x=x/(m->dim+1);
      y=y/(m->dim+1);
      z=z/(m->dim+1);
      flux(x,y,z,t,flx);
    }
  
}





static void set_mesh_volume(glRenderer rnd, GLSimplexMesh m)
{
  int i;
  if (m->xmax<m->xmin)
    {
      m->xmin=m->xmax=m->xc[1];
      m->ymin=m->ymax=m->yc[1];
      m->zmin=m->zmax=m->zc[1];
      for (i=2;i<=m->nn;i++)
	{
	  if (m->xc[i]<m->xmin) m->xmin=m->xc[i];
	  else if (m->xc[i]>m->xmax) m->xmax=m->xc[i];
	 
	  if (m->yc[i]<m->ymin) m->ymin=m->yc[i];
	  else if (m->yc[i]>m->ymax) m->ymax=m->yc[i];
	 
	  if (m->zc[i]<m->zmin) m->zmin=m->zc[i];
	  else if (m->zc[i]>m->zmax) m->zmax=m->zc[i];
	 
	}
    }
  
  if (m->dim==2) m->zmax=m->zmin+1;
  glrSetVolume(
	       m->rnd,  
	       m->xmin,m->xmax,
	       m->ymin,m->ymax,
	       m->zmin,m->zmax);

}





static void mesh_loop(glMesh glm, GLSimplexMesh m, glmSimplexCallback sxf)
{
  double coord0[3],coord1[3],coord2[3],coord3[3];
  double *coord[4];
  int i,in,*idx;
  int ncn=m->dim+1;
  double*xc=m->xc;
  double*yc=m->yc;
  double*zc=m->zc;
  int nc=m->nc;
  int imat=1;
  coord[0]=coord0;
  coord[1]=coord1;
  coord[2]=coord2;
  coord[3]=coord3;
 
  idx=m->sxlist+1;
  for (i=1;i<=nc;i++)
    {
      for(in=0;in<4;in++)
	{
	  coord[in][0]=xc[idx[in]];
	  coord[in][1]=yc[idx[in]];
	  coord[in][2]=zc[idx[in]];
	}
      sxf(glm,i,1,NULL,idx,coord);
      idx+=ncn;
    }
}

static int quit_application=0;

static int  do_quit_application(glRenderer rnd, int kbd_mod_mask)
{ 
  quit_application=1; 
  return GL_FALSE;    
}

void glsxvSetTitle(GLSimplexMesh m, char *title)
{
 if (m->rnd!=0) 
   {
     glrSetTitle(m->rnd,title);
     return;
   } 
}

static void glsxvCreateRenderer(GLSimplexMesh m)
{
 if (m->rnd) return;
 m->rnd=glrCreate("gltensview",0,0,500,500);
 set_mesh_volume(m->rnd,m);
 glrSetAxisName(m->rnd,'x',m->axisname[0]);
 glrSetAxisName(m->rnd,'y',m->axisname[1]);
 glrSetAxisName(m->rnd,'z',m->axisname[2]);
 glrSetAxisName(m->rnd,'f',m->axisname[3]);
 m->glm=glmCreate(m->nn,m->nc,m->dim,m,(glmLoopCallback)mesh_loop);
 glmSetVoffset(m->glm,1);
 glrSetVScale(m->rnd,1.0);
 glrSetInfoCallback(m->rnd,(glrDrawCallback)glmDrawInfo);
 glrRegisterKeyAction(m->rnd,GLW_q,do_quit_application,"q:Quit");
 glrSetDialog(m->rnd,0);
}

int glsxvData(GLSimplexMesh m)
{
  glsxvCreateRenderer(m);
  glrSetDialog(m->rnd,1);
  glmSetFunction(m->glm,m->nodefunc,1,-1);
  glRender(m->rnd,(glrDrawCallback)glmDraw,m->glm);       
  return !quit_application;
}

int glsxvNodeFuncf(GLSimplexMesh m,glsxvFunc f, double t, double fmin, double fmax)
{
  glsxvCreateRenderer(m);
  glsxvCalcNodeFunc(m,f,t);
  glmSetFunction(m->glm,m->nodefunc,fmin,fmax);
  glRender(m->rnd,(glrDrawCallback)glmDraw,m->glm);       
  return !quit_application;
}

int glsxvNodeFuncv(GLSimplexMesh m,double * f, double t, double fmin, double fmax)
{
  glsxvCreateRenderer(m);
  glrSetDialog(m->rnd,1);
  glmSetFunction(m->glm,f,fmin,fmax);
  glRender(m->rnd,(glrDrawCallback)glmDraw,m->glm);       
  return !quit_application;
}

int glsxvCellFluxf(GLSimplexMesh m,glsxvFlux f, double t, double fmin, double fmax)
{
  glsxvCreateRenderer(m);
  glsxvCalcCellFlux(m,f,t);
  glmSetCellFlux(m->glm,m->cellflux,fmin,fmax);
  glRender(m->rnd,(glrDrawCallback)glmDraw,m->glm); 
  return !quit_application;
}

int glsxvCellFluxv(GLSimplexMesh m,double *f, double t, double fmin, double fmax)
{
  glsxvCreateRenderer(m);
  glmSetCellFlux(m->glm,f,fmin,fmax);
  glRender(m->rnd,(glrDrawCallback)glmDraw,m->glm); 
  return !quit_application;
}

#define MAX_MESHES 100
static int initialized=0;
static GLSimplexMesh meshes[MAX_MESHES];

static int new_handle(GLSimplexMesh m)
{

  int i;
  if (!initialized)
    {
      for (i=1;i<MAX_MESHES;i++)
	meshes[i]=0;
      initialized=1;
    }

  for (i=1;i<MAX_MESHES;i++)
    {
      if (!meshes[i])
	{
	  meshes[i]=m;
	  return i;
	}
    }
  assert(0);
}

static GLSimplexMesh mesh(int i)
{
  assert(i>=0);
  assert(i<MAX_MESHES);
  assert(meshes[i]);
  return meshes[i];
}

static void del_handle(int i)
{
  assert(i>=0);
  assert(i<MAX_MESHES);
  assert(meshes[i]);
  meshes[i]=0;
}

void F77(glsxv_create)(int *dim,int *nn, int *nc, int *sxlist, double *xc,double *yc,double *zc, int *handle)
{
  GLSimplexMesh m;
  m=glsxvCreate(*dim,*nn,*nc,sxlist-1, xc-1,yc-1,zc-1);
  *handle=new_handle(m);
}

void F77(glsxv_box)(int *handle, 
		    double *xmin, double *xmax,
		    double *ymin, double *ymax,
		    double *zmin, double *zmax)
{

  GLSimplexMesh m;
  m=mesh(*handle);
  glsxvSetBox(m,*xmin,*xmax,*ymin,*ymax,*zmin,*zmax);
}
			    
void F77(glsxv_destroy)(int *handle)
{
  GLSimplexMesh m;
  m=mesh(*handle);
  if (m)
    glsxvDestroy(m);
  del_handle(*handle);
}

void F77(glsxv_title)(int *handle, char *title, int * l_title)
{
  char titbuf[128];
  int i;
  for (i=0;i< (*l_title); i++)
    titbuf[i]=title[i];
  glsxvSetTitle(mesh(*handle),titbuf);
}

void  F77(glsxv_nodefunc)(int *handle,double *f, double *t, double *fmin, double *fmax)
{
  GLSimplexMesh m;
  m=mesh(*handle);
  glsxvNodeFuncv(m,f-1, *t, *fmin, *fmax);
}

void  F77(glsxv_cellfflux)(int *handle,double *f, double *t, double *fmin, double *fmax)
{
  GLSimplexMesh m;
  m=mesh(*handle);
  glsxvCellFluxv(m,f-1, *t, *fmin, *fmax);
}

/*
 *  $Log: glsimplexview.c,v $
 *  Revision 1.3  2003/03/28 16:47:47  fuhrmann
 *  pdelib1.15_alpha1
 *
 *  Revision 1.2  2003/03/28 11:20:25  fuhrmann
 *  pdelib2.0_alpha1
 *
 *  Revision 1.1  2002/05/24 16:46:57  fuhrmann
 *  1. glsimplexview version an Klaus
 *
 *  Revision 2.5  2001/05/21 14:27:19  fuhrmann
 *  some warnings
 *  gltensview back from Gunther
 *
 *  Revision 2.3  2001/03/09 13:20:20  fuhrmann
 *   An Gunter
 *
 *  Revision 2.2  2001/03/08 17:37:28  fuhrmann
 *  First running version of gltensview stuff
 *
 */




